/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//
//OCO Source Materials
//
//5724-E76 and 5724-E77
//
// Copyright IBM Corp.  2003
//
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.faces.bf.validator;

import java.util.Stack;
import java.util.StringTokenizer;

import javax.servlet.jsp.tagext.PageData;
import javax.servlet.jsp.tagext.TagLibraryValidator;
import javax.servlet.jsp.tagext.ValidationMessage;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * <p>
 * A Custom validator for the tabbed panel. The validator checks the following condition:-
 * <ol>
 * <li> A tabbedPanel must contain one or more panels.</li>
 * <li> A tabbedPanel cannot have another tabbedPanel as a first child.</li>
 * <li> The tabbedPanel must have two JSF command buttons as the last two children (one finish button and one cancel button). In addition
 * the id of the buttons must be id of the tabbedPanel + "_finish and "_cancel".</li>
 * </ol>
 * </p>
 * @author fenil
 */
public class TabbedPanelValidator extends TagLibraryValidator {

	private static String OUTER_TAG = "tabbedPanel";
	private static String INNER_TAG = "bfPanel";
	private static String BUTTON_TAG = "commandExButton";
	private static String FACET_TAG = "facet";
	private static String ODC_URI = "http://www.ibm.com/jsf/BrowserFramework";
	private static String JSFEXT_URI = "http://www.ibm.com/jsf/html_extended";
	private static String JSFCORE_URI = "http://java.sun.com/jsf/core";
	private static String JSP_URI = "http://java.sun.com/JSP/Page";

	/* (non-Javadoc)
	 * @see javax.servlet.jsp.tagext.TagLibraryValidator#validate(java.lang.String, java.lang.String, javax.servlet.jsp.tagext.PageData)
	 */
	public ValidationMessage[] validate(String prefix, String uri, PageData page) {
		String jaxpPropertyName = "javax.xml.parsers.SAXParserFactory";
		// Pass the parser factory in on the command line with
		// -D to override the use of parser.
		if (System.getProperty(jaxpPropertyName) == null) {
			String apacheXercesPropertyValue = "org.apache.xerces.jaxp.SAXParserFactoryImpl";
			System.setProperty(jaxpPropertyName, apacheXercesPropertyValue);
		}
		DefaultHandler handler = new NestingHandler();
		SAXParserFactory factory = SAXParserFactory.newInstance();
		factory.setNamespaceAware(true);
		try {
			SAXParser parser = factory.newSAXParser();
			InputSource source = new InputSource(page.getInputStream());
			parser.parse(source, handler);
			return (null);
		} catch (Exception e) {
			String errorMessage = e.getMessage();
			// The first argument to the ValidationMessage
			// constructor can be a tag ID. Since tag IDs
			// are not universally supported, use null for
			// portability. The important part is the second
			// argument: the error message.
			ValidationMessage[] messages = { new ValidationMessage(null, errorMessage)};
			return (messages);
		}
	}
	
	/**
	 * A custom handler class to make the necessary validation checks on the tabbed panel.
	 * @author fenil
	 */
	class NestingHandler extends DefaultHandler {
		private boolean inOuterTag;
		private boolean innerTagSeen;
		private boolean inInnerTag;
		private Stack tabbedPanelStack;
		private boolean finishFacetSeen;
		private boolean cancelFacetSeen;
		private boolean backFacetSeen;
		private boolean nextFacetSeen;

		public NestingHandler() {
			this.inOuterTag = false;
			this.innerTagSeen = false;
			this.inInnerTag = false;
			tabbedPanelStack = new Stack();
		}
		
		/*
		 *  (non-Javadoc)
		 * @see org.xml.sax.ContentHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
		 */
		public void startElement(String namespaceUri, String localName, String qualifiedName, Attributes attributes)
			throws SAXException {
			String tagName = mainTagName(qualifiedName);
			//if there is any element after the finish and the cancel button and it is not an element belonging to JSP_URI for example a jsp:text element
			//which is automatically inserted then throw an exception
			if (!namespaceUri.equals(JSP_URI)
				&& this.inOuterTag
				&& !this.inInnerTag
				&& this.finishFacetSeen
				&& this.cancelFacetSeen
				&& this.backFacetSeen
				&& this.nextFacetSeen
				&& !tagName.equals(BUTTON_TAG)) {
				throw new SAXException("\nThe Last four elements in the tabbedpanel must be the back, next, finish and cancel facets.");
			}
			if (namespaceUri.equals(ODC_URI)) {
				if (tagName.equals(OUTER_TAG)) {
					//Cannot nest a tabbedpanel tag as a direct child of a tabbedpanel
					if (inOuterTag && !inInnerTag) {
						throw new SAXException("\nCannot nest " + OUTER_TAG + " in " + OUTER_TAG);
					}
					inOuterTag = true;
					String id = attributes.getValue("", "id");
					tabbedPanelStack.push(id);
					//if this is a nested tabbed panel then all the global vars need to be reset except inOuterTag which is already set above
					if (tabbedPanelStack.size() >= 1) {
						this.inInnerTag = false;
						this.innerTagSeen = false;
						this.finishFacetSeen = false;
						this.cancelFacetSeen = false;
						this.backFacetSeen = false;
						this.nextFacetSeen = false;

					}
				}
				//The panel tag has to be inside a tabbed panel. It cannot be used by itself 
				else if (tagName.equals(INNER_TAG) && !inOuterTag) {
					throw new SAXException("\n" + INNER_TAG + " can only appear within " + OUTER_TAG);
				}
				//set the flags once a panel tag is seen inside a tabbed panel 
				else if (tagName.equals(INNER_TAG)) {
					innerTagSeen = true;
					inInnerTag = true;
				}
			} else if (namespaceUri.equals(JSFEXT_URI)) {
				//make sure we are not looking at buttons inside a panel but at the buttons at the end of all the panels in the tabbed panel
				if (!inInnerTag && inOuterTag && tagName.equals(BUTTON_TAG)) {
					//id for the button
					/*String id = attributes.getValue("", "id");
					if (!id.equals(((String) tabbedPanelStack.peek()) + "_finish")
						&& !id.equals(((String) tabbedPanelStack.peek()) + "_cancel")
					    && !id.equals(((String) tabbedPanelStack.peek()) + "_back")
					    && !id.equals(((String) tabbedPanelStack.peek()) + "_next")) {
						throw new SAXException("\nInvalid id on button at the end of the tabbed panel");
					} else if (id.equals(((String) tabbedPanelStack.peek()) + "_finish")) {
						this.finishButtonSeen = true;
					} else if (id.equals(((String) tabbedPanelStack.peek()) + "_cancel")) {
						this.cancelButtonSeen = true;
					}else if (id.equals(((String) tabbedPanelStack.peek()) + "_back")) {
						this.backButtonSeen = true;
		            } else if (id.equals(((String) tabbedPanelStack.peek()) + "_next")) {
						this.nextButtonSeen = true;
					}*/
					//The button must be inside one of the facets
					if(!finishFacetSeen || !cancelFacetSeen || !backFacetSeen || !nextFacetSeen){
							throw new SAXException("\nButtons must be inside a facet.");
					}
				}

			}
			else if(namespaceUri.equals(JSFCORE_URI) && tagName.equals(FACET_TAG)){
				String name = attributes.getValue("","name");	
				if(name.equals("finish")){
					this.finishFacetSeen = true;	
				}
				else if(name.equals("cancel")){
					this.cancelFacetSeen = true;
				}
				else if(name.equals("back")){
					this.backFacetSeen = true;
				}
				else if(name.equals("next")){
					this.nextFacetSeen = true;
				}
			}
		}
		
		/*
		 *  (non-Javadoc)
		 * @see org.xml.sax.ContentHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
		 */
		public void endElement(String namespaceUri, String localName, String qualifiedName) throws SAXException {
			String tagName = mainTagName(qualifiedName);
			if (namespaceUri.equals(ODC_URI)) {
				if (tagName.equals(OUTER_TAG)) {
					inOuterTag = false;
					//The tabbed panel has to have atleast one panel inside it
					if (!innerTagSeen) {
						throw new SAXException("\n" + OUTER_TAG + " must contain atleast one " + INNER_TAG);
					}
					if(!this.finishFacetSeen && !this.cancelFacetSeen && !this.backFacetSeen && !this.nextFacetSeen){
						throw new SAXException("\nThe finish, cancel, back and next facets must not be moved, deleted or renamed.");
					}
					this.finishFacetSeen = false;
					this.cancelFacetSeen = false;
					this.backFacetSeen = false;
					this.nextFacetSeen = false;
					tabbedPanelStack.pop();
					//if there is one or more objects in the stack then we are inside a nested tabbed panel then this means that inOuterTag is true and a panel has already been seen
					if (tabbedPanelStack.size() >= 1) {
						this.inOuterTag = true;
						this.inInnerTag = true;
						this.innerTagSeen = true;
					}
				} else if (tagName.equals(INNER_TAG)) {
					inInnerTag = false;
				}
			}
		}
		
		/**
		 * Utility method to get the unqualifed name from a qualified token.
		 * @param qualifiedName the qualified name.
		 * @return the unqualified name.
		 */
		private String mainTagName(String qualifiedName) {
			StringTokenizer tok = new StringTokenizer(qualifiedName, ":");
			tok.nextToken();
			return (tok.nextToken());
		}

	}
}
