/************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corp. 2003.  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 * sample code created by IBM Corporation.  This sample code is
 * not part of any standard or IBM product and is provided to you
 * solely for the purpose of assisting you in the development of
 * your applications.  The code is provided "AS IS", without
 * warranty of any kind.  IBM shall not be liable for any damages
 * arising out of your use of the sample code, even if they have
 * been advised of the possibility of such damages.
 *************************************************************************/
package com.ibm.odcb.jrender.diff;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EReference;
import com.ibm.odcb.jrender.mediators.gen.ecore.EFeatureMap;
import com.ibm.odcb.jrender.misc.EMFHelper;
import com.ibm.odcb.jrender.misc.Streamer;
import java.util.HashMap;
import java.util.Iterator;
import java.util.ArrayList;
import com.ibm.odcb.jrender.mediators.Mediator;

////#IF# SDO
//import commonj.sdo.DataObject;
////#END#

////#IF# WDO
import com.ibm.etools.wdo.DataObject;
////#END#

/**
 * @author Titania Gupta
 * This class is used to traverse the Diff and handle the changes made to the objects in the Diff.
 * Following are the steps to handle the changes in the Diff objects:
 * 1.  Instantiate DiffTraverser with either the Diff EObject 
 *     or the String represeting the Diff along with the name of the modelRoot object.
 * 2.  Set <code>DiffHandler<code>s for each type of EObject in the Diff
 *     by using the addHandler(eClassName, DiffHandler) method.  
 *     The <code>DiffHandler<code>s are application specific custom handlers you implement 
 *     to handle/persist the changes made to the objects in the Diff.  Saving the changes to a database is an Example.
 * 3.  Call DiffTraverse.traverse() method.  The DiffTraverser traverses the EObjects in the Diff in their 'Current' state
 *     and calls their respective DiffHandlers with it's <code>DiffInfo<code> to 'handle' any updates (Create, Update or Delete actions).
 * @see <code>DiffHandler<code>
 * @see <code>DiffInfo<code>
 */
public class DiffTraverser
{
	  /**
	   * Constructor for DiffTraverser.
	   * @param modelRootName The name of the root element of the data model of the Diff.
	   *                       This is the name of the root child element of the 'Current' node of the Diff.
	   * @param theDiff   The String represention of the Diff.
	   */
	  public DiffTraverser(String modelRootName, String theDiffString) throws Exception
	  {
	    this.diff = (EObject) Mediator.getDiffDataObject(modelRootName, theDiffString);
	    if (this.diff == null)
	      throw new Exception("The Diff DataObject could not be obtained with the given parameters.  \nPlease validate your modelRootName and theDiffString and try again.");
	  }

  /**
   * @deprecated
   * Constructor for DiffTraverser.
   * @param theDiff - The org.eclipse.emf.ecore.EObject represention of the Diff.
   */
  public DiffTraverser(EObject theDiff)
  {
    this.diff = theDiff;
  }
  
  public DiffTraverser(DataObject diffDataObject)
  {
  	this.diff = (EObject) diffDataObject;
  }
   
  protected EObject diff;
  //protected DataObject diff;
  protected EObject current;
  protected EObject original;
  protected HashMap handlers = new HashMap(7);
  /**
   * Sets the DiffHandler for the objects of the given eClassName type.
   * @param eClassName The name of the EClassifier of the object.
   *                    This is the export name on the EClassMap for the object. 
   *                    This is also the EClassifier name of the object in the generated XX_client.ecore file.
   * @param handler    The application specific custom handler used to handle/persist the changes made 
   *                    to the objects of the given eClassName type in the Diff.
   *                    Activities like validating and saving these changes to a database 
   *                    are implemented using these <code>DiffHandler<code>s.
   * @see   <code>DiffHandler<code>
   * @see   <code>DiffInfo<code>
   */ 
  public void addHandler(String eClassName, DiffHandler handler)
  {
    handlers.put(eClassName, handler);
  }
  
  /**
   * Gets the DiffHandler for the given eClassName.
   * @param  eClassName The name of the EClassifier of the object.
   *                     This is the export name on the EClassMap for the object. 
   *                     This is also the EClassifier name of the object in the generated XX_client.ecore file.
   * @return the DiffHandler for the given eClassName.
   */
  public DiffHandler getHandler(String eClassName)
  {
    return (DiffHandler) handlers.get(eClassName);
  }
  
  /**
   * Traverses the Diff associated with this DiffTraverser instance by calling the traverseCurrentDiff() method
   * which handles the changes (Create, Update, Delete) to the objects contained
   * within the Diff using the <code>DiffHandler<code>s set prior to calling this method.
   * If an object has changes pending, and a DiffHandler is not found for the object,
   * an Exception is thrown.
   * @exception java.lang.Exception is thrown if an object has changes pending, and a DiffHandler is not found for the object.
   */
  public void traverse() throws Exception 
  {
    if (diff == null)
      throw new Exception("Cannot process Diff.  The Diff EObject is null!");
      
    current  = (EObject) EMFHelper.getEFeatureValue(diff, DiffHelper.CURRENT_EFEATURE_NAME);
    original = (EObject) EMFHelper.getEFeatureValue(diff, DiffHelper.ORIGINAL_EFEATURE_NAME);

    if (current == null)//original can be null if CRUD action is D (Delete) or C (create).
      throw new Exception("Diff's 'Current' EFeature cannot be null!");
      
    traverseCurrentDiff();
  }
  
  /**
   * Traverses the Diff associated with this DiffTraverser instance
   * and handles the changes (Create, Update, Delete) to the objects contained
   * within the Diff using the <code>DiffHandler<code>s set prior to calling this method.
   * <p>
   * This method is usually called by the traverse() method.  Users should call the traverse() method.
   * If using traverseCurrentDiff() directly, you must first set the Current and Original nodes of the Diff
   * by calling the setCurrent(current) and setOriginal(original) methods.
   * <p>
   * If an object has changes pending, and a DiffHandler is not found for the object,
   * an Exception is thrown.
   * @exception java.lang.Exception is thrown if an object has changes pending, and a DiffHandler is not found for the object.
   */
  public void traverseCurrentDiff() throws Exception
  {
    EClass c = current.eClass();
    Iterator refs = c.getEReferences().iterator();
    DiffInfo parent = null;
    ArrayList handledObjects = new ArrayList(7);
    while (refs.hasNext())
     {
       EReference ref = (EReference) refs.next();
       Object refobj = current.eGet(ref, true);
       if (refobj != null)
        {
          if (refobj instanceof org.eclipse.emf.ecore.util.EcoreEList)
           {
             handleEObjects( (org.eclipse.emf.ecore.util.EcoreEList) refobj, parent, handledObjects, ref.getName() );
           }
          else if (refobj instanceof EObject)
           {
             handleEObject( (EObject) refobj, parent, handledObjects, ref.getName() );              
           }
        }
     }
  }
  
  /**
   * Calls the DiffHandler for the given EObject.
   * It then iterates through the given EObject's EReferences and
   * recursively calls handleEObject() or handleEObjects() depending on it's mulitplicity.
   * This method is usually called from within the handleCurrentDiff() method.
   * @param o              The EObject
   * @param ancestors      The list of ancestors of the EObject.  This is empty the first time
   *                        this method is called.  The same list is passed to subsequent calls to this method.
   * @param handledObjects The list of objects already handled. This is empty the first time
   *                        this method is called.  The same list is passed to subsequent calls to this method.
   *                        This is needed to avoid infinite loops caused by two way references.
   * @see   handleEObjects(org.eclipse.emf.ecore.util.EcoreDynamicEList, Vector, Vector)
   */
  public void handleEObject(EObject o, DiffInfo parent, ArrayList handledObjects, String propertyName) throws Exception
  {
    if (o == null)
      return;
           
    if (handledObjects == null)
      handledObjects = new ArrayList(7);
      
    //if this EObject has already been printed then skip.  This is needed to avoid infinite loops.
    boolean handled = handledObjects.contains(o);
    if (!handled)  
    {         
      handledObjects.add(o);
      
      EClass c = o.eClass();
      char crud = ((Character) EMFHelper.getEFeatureValue(o, EFeatureMap.ODYSSEY_STATUS_FEATURE)).charValue();
      DiffHandler h = getHandler(c.getName());

      //get odc:id feature.  odc:id is same as xmi:id.  xmi:id cannot be obtained from EObject.  EMF does not include this in the list of eattributes
      String id = (String) EMFHelper.getEFeatureValue( o, EFeatureMap.ODYSSEY_ID_FEATURE );
	  if (h == null)
	  {
	   Streamer.warning.Header().println("DiffHandler for '"+c.getName()+"' is NULL.  Using the default do-nothing handler instead.  If you made changes to '"+c.getName()+"'s' attributes they will not be saved!  If you have assigned handlers for it's references, those will be handled.");
	   h = new BaseHandler();
	  }

	  //set up original object
	  EObject orig;      
      if (crud == 'C')
      {
		orig = null;
      }
      else if (crud == 'D')
      {
		orig = o;
      }
      else
      {
		orig = DiffHelper.getOriginalEObjectByXmiId(diff, id);
      }
      
      if (crud == 'U' && orig == null)
      {
          throw new Exception("The Original EObject for '"+c.getName()+"' with xmi:id '"+id+"' IS NULL");
      }
      
      DiffInfo di = new DiffInfo(c.getName(), crud, (DataObject) o, (DataObject) orig, parent, propertyName);
	  h.handle(di);           
      //set up the parent for next round
      parent = di;

      //check child references to see if they have been modified
      Iterator refs = c.getEReferences().iterator();
      while (refs.hasNext())
      {
         EReference ref = (EReference) refs.next();
         Object refobj = o.eGet(ref, true);
         if (refobj != null)
         {
           if (refobj instanceof org.eclipse.emf.ecore.util.EcoreEList)
           {
             handleEObjects( (org.eclipse.emf.ecore.util.EcoreEList) refobj, parent, handledObjects, ref.getName());
           }
           else if (refobj instanceof EObject)
           {
             handleEObject((EObject) refobj, parent, handledObjects, ref.getName());
           }
         }
       }
       
     }
   }

  /**
   * Iterates through the given list's EObjects or Lists and
   * recursively calls handleEObject() or handleEObjects() respectively to handle changes to them if any.
   * This method is usually called from within the handleCurrentDiff(), handleEObject() or handleEObjects() methods.
   * @param list           The list containing EObjects or Lists.
   *                        This usually represents an EReference object of multiplicity >1.
   * @param ancestors      The list of ancestors of the EObject.  This is empty the first time
   *                        this method is called.  The same list is passed to subsequent calls to this method.
   * @param handledObjects The list of objects already handled. This is empty the first time
   *                        this method is called.  The same list is passed to subsequent calls to this method.
   *                        This is needed to avoid infinite loops caused by two way references.
   * @see   handleEObject(org.eclipse.emf.ecore.EObject, Vector, Vector)
   
  public void handleEObjects(org.eclipse.emf.ecore.util.EcoreDynamicEList list, Vector ancestors, Vector handledObjects) throws Exception
  {
    Iterator it = list.iterator();
    while (it.hasNext())
    {
      Object eobj = it.next();
      if (eobj instanceof org.eclipse.emf.ecore.util.EcoreDynamicEList)
      {
        handleEObjects( (org.eclipse.emf.ecore.util.EcoreDynamicEList) eobj, ancestors, handledObjects);
      }
      else if (eobj instanceof EObject)
      {
        handleEObject((EObject)eobj, ancestors, handledObjects);
      }
    }
  }*/
  
  /**
   * Iterates through the given list's EObjects or Lists and
   * recursively calls handleEObject() or handleEObjects() respectively to handle changes to them if any.
   * This method is usually called from within the handleCurrentDiff(), handleEObject() or handleEObjects() methods.
   * @param list           The list containing EObjects or Lists.
   *                        This usually represents an EReference object of multiplicity >1.
   * @param ancestors      The list of ancestors of the EObject.  This is empty the first time
   *                        this method is called.  The same list is passed to subsequent calls to this method.
   * @param handledObjects The list of objects already handled. This is empty the first time
   *                        this method is called.  The same list is passed to subsequent calls to this method.
   *                        This is needed to avoid infinite loops caused by two way references.
   * @see   handleEObject(org.eclipse.emf.ecore.EObject, Vector, Vector)
   */
  public void handleEObjects(org.eclipse.emf.ecore.util.EcoreEList list, DiffInfo parent, ArrayList handledObjects, String propertyName) throws Exception
  {
    Iterator it = list.iterator();
    while (it.hasNext())
    {
      Object eobj = it.next();
      if (eobj instanceof org.eclipse.emf.ecore.util.EcoreEList)
      {
        handleEObjects( (org.eclipse.emf.ecore.util.EcoreEList) eobj, parent, handledObjects, propertyName);
      }
      else if (eobj instanceof EObject)
      {
        handleEObject((EObject)eobj, parent, handledObjects, propertyName);
      }
    }
  }  
  
  /**
   * Returns the Current node of the Diff.
   * @return EObject
   */
  public EObject getCurrent()
  {
    return current;
  }

  /**
   * Returns the Diff.
   * @return EObject
   */
  public EObject getDiff()
  {
    return diff;
  }

  /**
   * Returns the handlers.
   * @return HashMap
   */
  public HashMap getHandlers()
  {
    return handlers;
  }

  /**
   * Returns the Original node of the Diff.
   * @return EObject
   */
  public EObject getOriginal()
  { 
    return original;
  }

  /**
   * Sets the handlers.
   * @param handlers The handlers to set
   */
  public void setHandlers(HashMap handlers)
  {
    this.handlers = handlers;
  }

  /**
   * Sets the Current node of the Diff.
   * @param current The current to set
   */
  public void setCurrent(EObject current)
  {
    this.current = current;
  }

  /**
   * Sets the Original node of the Diff.
   * @param original The original to set
   */
  public void setOriginal(EObject original)
  {
    this.original = original;
  } 

}
