/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2003
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.odcb.jrender.emitters;

import java.io.IOException;
import java.io.Writer;

import com.ibm.odcb.jrender.mediators.ExportException;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.Config;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.misc.URLRewriter;
import java.util.ArrayList;

import com.ibm.odcb.jrender.misc.StringUtil;
/**
 * The <strong>GraphDrawEmitter</strong> generates code to render a chart (bar, pie or line) and tie it to a 
 * data model on a page. The chart is a Macromedia Flash object.
 * 
 * @author Kaushal Kurapati
 * @author Vidya Rao
 */
public class GraphDrawEmitter extends BaseEmitter {
	public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;
	private static final String EMFGRAPH2_SWF = Config.versionizeFilename("emfChart2.swf");

	private static final String GRAPHDRAWCONTROL = Config.versionizeFilename("graphdrawcontrol.js");
	private static final String GRAPHDRAWADAPTER = Config.versionizeFilename("graphdrawadapter.js");
	private static final String JSL_GRAPHDRAW_C = Config.versionizeFilename("jsl_graphdraw_c.js");
	private static final String ECLASS = Config.versionizeFilename("eclass.js");
	private static final String ECREATOR = Config.versionizeFilename("ecreator.js");
	private static final String EOBJECT = Config.versionizeFilename("eobject.js");
	private static final String XMIHANDLER = Config.versionizeFilename("xmiHandler.js");
	private static final String BINDER = Config.versionizeFilename("binder.js");
	private static final String XML2EMF = Config.versionizeFilename("xml2emf.js");
	
	private static final String JSL_WDO4JS_C = Config.versionizeFilename("jsl_wdo4js_c.js");
	
	
	private boolean _isPageData = false;
	/**
	 * The default constructor only calls the BaseEmitter's constructor. 
	 */
	public GraphDrawEmitter() {
		super("ODCGraphDraw");
	}

	/**
	 * This constructor calls the Init() method with the input parameters. 
	 * 
	 * @param id 				The Id of the control
	 * @param modelName 		The name of the model that the control is bound to
	 * @param eObjectId	 		The eObjectId of the data object that the control is bound to
	 * @param eReferenceName	The reference name of the data object
	 * @param width 			The width of the chart, in pixels (400) or as a percentage (40%) 
	 * @param height 			The height of the chart, in pixels (300) or as a percentage (30%)
	 * @param firstChart 		The chart type that is displayed by default when the graph is rendered 
	 * 							Can be one of "Pie", "Bar", "Line"
	 * 							If not specified, defaults to "Bar"
	 * @param showPie 			If true, pie chart is one of the available chart types. Defaults to false.
	 * @param showBar 			If true, bar chart is one of the available chart types. Defaults to false.
	 * @param showLine 			If true, line chart is one of the available chart types. Defaults to false.
	 * @param yAxisDivisions 	The number of divisions into which the y axis is split. Defaults to 10.
	 * @param title				The title of the graph
	 * @param xAxisTitle		The x axis title
	 * @param yAxisTitle		The y axis title
	 * @param showLegend		If true, the legend of series names is displayed. Defaults to true.
	 * @param splitYAxis		If true, in the case of a graph with exactly two series, there are two y axis
	 * 							plotted, one on each side of the x axis. Defaults to false.
	 */
	public GraphDrawEmitter(
		String id,
		String modelName,
		String eObjectId,
		String eReferenceName,
		String width,
		String height,
		String firstChart,
		boolean showPie,
		boolean showBar,
		boolean showLine,
		String yAxisDivisions,
		String title,
		String xAxisTitle,
		String yAxisTitle,
		boolean showLegend,
		boolean splitYAxis,
		boolean showLabel,
		boolean showHBar,
		boolean showHLine,
		boolean showVLine,
		String styleClass) {		
		super("ODCGraphDraw");
		this.Init(
			id,
			modelName,
			eObjectId,
			eReferenceName,
			width,
			height,
			firstChart,
			showPie,
			showBar,
			showLine,
			yAxisDivisions,
			title,
			xAxisTitle,
			yAxisTitle,
			showLegend,
			splitYAxis,
			showLabel,
			showHBar,
			showHLine,
			showVLine,
			styleClass);			
	}

	public void setIsPageData(boolean ipd)
	{
		this._isPageData = ipd;
	}
	
	
	/**
	 * Initializes corresponding class variables with the values of the input parameters.
	 * This is called by the multi-argument constructor. 
	 *  
	 * @param id 				The Id of the control
	 * @param modelName 		The name of the model that the control is bound to
	 * @param eObjectId	 		The eObjectId of the data object that the control is bound to
	 * @param eReferenceName	The reference name of the data object
	 * @param width 			The width of the chart, in pixels (400) or as a percentage (40%) 
	 * @param height 			The height of the chart, in pixels (300) or as a percentage (30%)
	 * @param firstChart 		The chart type that is displayed by default when the graph is rendered 
	 * 							Can be one of GraphDrawEmitter.PIE_CHART, GraphDrawEmitter.BAR_CHART, GraphDrawEmitter.LINE_CHART
	 * 							If not specified, defaults to GraphDrawEmitter.BAR_CHART
	 * @param showPie 			If true, pie chart is one of the available chart types. Defaults to false.
	 * @param showBar 			If true, bar chart is one of the available chart types. Defaults to false.
	 * @param showLine 			If true, line chart is one of the available chart types. Defaults to false.
	 * @param yAxisDivisions 	The number of divisions into which the y axis is split. Defaults to 10.
	 * @param title				The title of the graph
	 * @param xAxisTitle		The x axis title
	 * @param yAxisTitle		The y axis title
	 * @param showLegend		If true, the legend of series names is displayed. Defaults to true.
	 * @param splitYAxis		If true, in the case of a graph with exactly two series, there are two y axis
	 * 							plotted, one on each side of the x axis. Defaults to false.
	 */
	public void Init(
		String id,
		String modelName,
		String eObjectId,
		String eReferenceName,
		String width,
		String height,
		String firstChart,
		boolean showPie,
		boolean showBar,
		boolean showLine,
		String yAxisDivisions,
		String title,
		String xAxisTitle,
		String yAxisTitle,
		boolean showLegend,
		boolean splitYAxis,
		boolean showLabel,
		boolean showHBar,
		boolean showHLine,
		boolean showVLine,
		String styleClass) {		
		this.id = id;
		this.modelName = modelName;
		this.eObjectId = eObjectId;
		this.eReferenceName = eReferenceName;
		this.width = width;
		this.height = height;
		this.firstChart = firstChart;
		this.showPie = showPie;
		this.showBar = showBar;
		this.showLine = showLine;
		this.yAxisDivisions = yAxisDivisions;
		if (this.yAxisDivisions == null || this.yAxisDivisions.equalsIgnoreCase("auto")) {
			Streamer.debug.Header().println("Setting the auto option for y axis divisions " + this.yAxisDivisions);
			this.yAxisDivisions = "-1";
		}
		this.title = title;
		this.xAxisTitle = xAxisTitle;
		this.yAxisTitle = yAxisTitle;
		this.showLegend = showLegend;
		this.splitYAxis = splitYAxis;
		this.showLabel = showLabel;
		this.showHBar = showHBar;
		this.showHLine = showHLine;
		this.showVLine = showVLine;
		this.styleClass = styleClass;		

		Streamer.debug.Header().println(
			"Initialized GraphDrawEmitter with these parameters \n"
				+ "id = "
				+ id
				+ " ,"
				+ "modelName = "
				+ modelName
				+ " ,"
				+ "eObjectId = "
				+ eObjectId
				+ " ,"
				+ "eReferenceName = "
				+ eReferenceName
				+ " ,"
				+ "width = "
				+ width
				+ " ,"
				+ "height = "
				+ height
				+ " ,"
				+ "firstChart = "
				+ firstChart
				+ " ,"
				+ "showPie = "
				+ showPie
				+ " ,"
				+ "showBar = "
				+ showBar
				+ " ,"
				+ "showLine = "
				+ showLine
				+ " ,"
				+ "yAxisDivisions = "
				+ yAxisDivisions
				+ " ,"
				+ "title = "
				+ title
				+ " ,"
				+ "xAxisTitle = "
				+ xAxisTitle
				+ " ,"
				+ "yAxisTitle = "
				+ yAxisTitle
				+ " ,"
				+ "showLegend = "
				+ showLegend
				+ " ,"
				+ "splitYAxis = "
				+ splitYAxis
				+ " ,"
				+ "showHBar = "
				+ showHBar
				+ " ,"
				+ "showHLine = "
				+ showHLine
				+ " ,"
				+ "showVLine = "
				+ showVLine
				+ " ,"
				+ "styleClass = "
				+ styleClass
				);				
	}

	
	
	
	
	public void exportDIV(Writer out, PageContext PC) throws ExportException, IOException {
		
		
		URLRewriter R = (URLRewriter) PC.getContextVariable(InitializationEmitter._ID_URL_REWRITER);
		
		_varIndex = PC.NextInstanceCount();
		
		boolean isFaces=StringUtil.checkEncoding();
		String encodedAdapterVar = StringUtil.encodeString(ADAPTER_VAR+ _varIndex,isFaces);
		String encodedControlVar = StringUtil.encodeString(CONTROL_VAR+ _varIndex,isFaces);
		
		// These dummy objects are created to keep flash 8 in IE happy
		out.write("<script type=\"text/javascript\">var "+StringUtil.encodeString(CONTROL_VAR+ _varIndex,isFaces)+"Chart = new Object();\n");
		out.write(StringUtil.encodeString(CONTROL_VAR+ _varIndex,isFaces)+"Chart.SetReturnValue = function() {} \n ");
		out.write("var "+StringUtil.encodeString(CONTROL_VAR+ _varIndex,isFaces)+" = new Object();");
		out.write(StringUtil.encodeString(CONTROL_VAR+ _varIndex,isFaces)+".getGraphVariables = new function() {}  </script>\n");
		
		
		out.write(
				"<div style=\"position: absolute; top:0px;left:0px; width:0px; height:0px; overflow:hidden\" id=\""
					+ encodedAdapterVar
					+ "_AccessibleText\"></div>");

			out.write(
				//"<object classid=\"clsid:D27CDB6E-AE6D-11cf-96B8-444553540000\" codebase=\"http://download.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=7,0,0,0\" id=\""
					"<object id=\""
					+ encodedControlVar
					//+ "Chart\" width=\""
					+ "Chart\" type=\"application/x-shockwave-flash\" data=\"" + R.Rewrite("jsl/graphdraw/" + EMFGRAPH2_SWF) + "\" width=\""
					+ width
					+ "\" height=\""
					+ height
					//+ "\" ALIGN=\"VIEWASTEXT\" VIEWASTEXT>");
					+ "\">");
			out.write("\n");

			//params
			out.write("<param name=\"allowScriptAccess\" value=\"always\" />");
			out.write("\n");
			out.write("<param name=\"flashvars\" value=\"callBackFunc="+ encodedControlVar+".getGraphVariables\" />");
			out.write("\n");
			out.write("<param name=\"movie\" value=\"" + R.Rewrite("jsl/graphdraw/" + EMFGRAPH2_SWF) + "\" />");
			out.write("\n");

			//embed
			/*out.write(
				"<EMBED src=\""
					+ R.Rewrite("jsl/graphdraw/emfChart2.swf")
					+ "\" menu=\"false\" quality=\"high\" bgcolor=\"#FFFFFF\" WIDTH=\""
					+ width
					+ "\" HEIGHT=\""
					+ height
					+ "\" swLiveConnect=\"true\" NAME=\""
					+ CONTROL_VAR
					+ _varIndex
					+ "Chart\" ALIGN=\"\" TYPE=\"application/x-shockwave-flash\" PLUGINSPAGE=\"http://www.macromedia.com/go/getflashplayer\"></EMBED>");*/
			out.write("\n");
			out.write("</object>");
			out.write("\n");
		
	}
	
	
	
	/**
	 * Exports the JavaScript code onto the page. 
	 * It creates the Graph control and adapter, binds the control to the parent object, and initializes 
	 * it with all specified parameters. It finally calls a method on the Graph control that renders the 
	 * graph content.
	 * <p>
	 * @param out	Writer used to write the JavaScript
	 * @param PC	PageContext that supplies the instance count to uniquely instantiate variables.
	 */
	public void Export(Writer out, PageContext PC) throws ExportException, IOException {
		Streamer.trace.Header().println("Entering GraphDrawEmitter...");
	//	out.write("<script type=\"text/javascript\">\n");
		out.write("ODCProgressBar.startRenderControl(graphdraw);\n");
		out.write("</script>\n");
	//	_varIndex = PC.NextInstanceCount();
		URLRewriter R = (URLRewriter) PC.getContextVariable(InitializationEmitter._ID_URL_REWRITER);
		boolean debugMode =
			PC.getContextVariable(InitializationEmitter._ID_DEBUG_MODE) == InitializationEmitter.DEBUG_YES
				? true
				: false;

		if (TestGuard(PC, "GRAPH_JS_FLAG") == false) {
			if (debugMode == true) {
				out.write(
					"\n<script type=\"text/javascript\" src=\""
						+ R.Rewrite("jsl/graphdraw/" + GRAPHDRAWCONTROL)
						+ "\"></script>"
						+ "\n<script type=\"text/javascript\" src=\""
						+ R.Rewrite("jsl/graphdraw/" + GRAPHDRAWADAPTER)
						+ "\"></script>");
				Streamer.trace.Header().println("Included graphdraw JS control and adapter files...");
				if (this._isPageData)
				{
					out.write("\n<script type=\"text/javascript\" src=\"" + R.Rewrite("jsl/" +  ECLASS) + "\"></script>\n<script type=\"text/javascript\" src=\""
			                + R.Rewrite("jsl/" + ECREATOR) + "\"></script>\n<script type=\"text/javascript\" src=\"" + R.Rewrite("jsl/" + EOBJECT)
			                + "\"></script>\n<script type=\"text/javascript\" src=\"" + R.Rewrite("jsl/" + XMIHANDLER)
			                + "\"></script>\n<script type=\"text/javascript\" src=\"" + R.Rewrite("jsl/" + BINDER) + "\"></script>\n");
					out.write(
							"<script type=\"text/javascript\" src=\"" + R.Rewrite("jsl/" + XML2EMF) + "\"></script>");
						out.write("\n");
				}
				
				
			} else {
				out.write(
					"<script type=\"text/javascript\" src=\"" + R.Rewrite("jsl/" + JSL_GRAPHDRAW_C) + "\"></script>");
				out.write("\n");
				Streamer.trace.Header().println("Included compressed graphdraw JS file...");
				if (this._isPageData)
				{
					out.write("<script type=\"text/javascript\" src=\"" + R.Rewrite("jsl/" + JSL_WDO4JS_C) + "\"></script>");
						out.write("\n");
				}
				
			}
			
		}
		
		

		
		
		
		//Check should encoding occur
		boolean isFaces=StringUtil.checkEncoding();
			
		String encodedAdapterVar = StringUtil.encodeString(ADAPTER_VAR+ _varIndex,isFaces);
		String encodedControlVar = StringUtil.encodeString(CONTROL_VAR+ _varIndex,isFaces);
		modelName = StringUtil.encodeString(modelName,isFaces);
		eObjectId = StringUtil.encodeString(eObjectId,isFaces);
		
		String xmlObjName = null;
		String modelRoot = null;
		
		if (this._isPageData)
		{
			modelRoot = modelName;
			xmlObjName = modelRoot+"_xml2emf";
		}
		else 
		{
			modelRoot = StringUtil.encodeString("WDO4JSModelRoot_"
					+ modelName + ".Root",isFaces); 
		}
		

		out.write("<script type=\"text/javascript\">");
		
		if (this._isPageData)
		{
			out.write("var "+xmlObjName+" = new XML2EMF("+modelRoot+");\n");
		}
		
		
		//create the graphdrawcontrol object
		out.write("var " + encodedControlVar + " = new GraphDrawControl();");
		out.write("\nwindow."+encodedControlVar + " = "+encodedControlVar+";");
		out.write("\n");
		out.write(encodedControlVar + ".chartObj = document.getElementById('" + encodedControlVar + "Chart');");
		out.write("\n");
		out.write("window." + encodedControlVar +"Chart = "+encodedControlVar + ".chartObj");
		out.write("\n");

		//create adapter
		out.write("var " + encodedAdapterVar + " = new GraphDrawAdapter(" + encodedControlVar + ");");
		out.write("\n");
		
		if (this._isPageData)
		{

			out.write(
					encodedAdapterVar
					+ ".parentEObject = "
					+ xmlObjName);
			out.write(";\n");
		}
		else 
		{	
			out.write(
					encodedAdapterVar
					+ ".parentEObject = findEObjectByXMIID("
					+ modelRoot
					+ ", \""
					+ eObjectId
					+ "\");");
			out.write("\n"); 
		}

		//point and label data
		out.write(encodedAdapterVar + ".propertyName = \"" + eReferenceName + "\";");
		out.write("\n");
		out.write(encodedAdapterVar + "." + LABEL_ATTRIBUTE_PARAM + " = \"" + labelAttributeName + "\";");
		out.write("\n");

		processDataSeries();

		//series names and labels		
		out.write(encodedControlVar + "." + SERIES_NAMES_PARAM + " = new Array(" + seriesNames.toString() + ");");
		out.write("\n");
		out.write(
			encodedAdapterVar + "." + VALUE_ATTRIBUTES_PARAM + " = new Array(" + attNames.toString() + ");");
		out.write("\n");
		out.write(encodedAdapterVar + "." + GROUP_OPS_PARAM + " = new Array(" + ops.toString() + ");");
		out.write("\n");

		out.write(encodedAdapterVar + "." + LABEL_FORMAT_PARAM + " = \"" + labelFormat + "\";");
		out.write("\n");

		out.write(encodedAdapterVar + "." + DATA_FORMAT_PARAM + "  = \"" + dataFormat + "\";");
		out.write("\n");

		out.write(encodedAdapterVar + "." + LABEL_CONVERTER_PARAM + " = " + labelConverterScript + ";");
		out.write("\n");

		out.write(encodedAdapterVar + "." + DATA_CONVERTER_PARAM + " = " + dataConverterScript + ";");
		out.write("\n");

		//all flags. titles
		out.write(encodedControlVar + "." + CHART_TITLE_PARAM + " = \"" + title + "\";");
		out.write("\n");
		out.write(encodedControlVar + "." + X_AXIS_TITLE_PARAM + " = \"" + xAxisTitle + "\";");
		out.write("\n");
		out.write(encodedControlVar + "." + Y_AXIS_TITLE_PARAM + " = \"" + yAxisTitle + "\";");
		out.write("\n");
		out.write(encodedControlVar + "." + SHOW_LEGEND_PARAM + " = " + showLegend + ";");
		out.write("\n");
		out.write(encodedControlVar + "." + SHOW_LABEL_PARAM + " = " + showLabel + ";");
		out.write("\n");
		out.write(encodedControlVar + "." + SHOW_PIE_PARAM + " = " + showPie + ";");
		out.write("\n");
		out.write(encodedControlVar + "." + SHOW_BAR_PARAM + " = " + showBar + ";");
		out.write("\n");
		out.write(encodedControlVar + "." + SHOW_LINE_PARAM + " = " + showLine + ";");
		out.write("\n");
		out.write(encodedControlVar + "." + FIRST_CHART_PARAM + " = \"" + firstChart + "\";");
		out.write("\n");
		out.write(encodedAdapterVar + "." + Y_AXIS_DIVISIONS_PARAM + " = " + yAxisDivisions + ";");
		out.write("\n");
		out.write(encodedControlVar + "." + SPLIT_Y_AXIS_PARAM + " = " + splitYAxis + ";");
		out.write("\n");
		out.write(encodedAdapterVar + "." + GROUPED_PARAM + " = " + grouped + ";");
		out.write("\n");
		out.write(encodedControlVar + "." + SHOW_H_BAR_PARAM + " = " + showHBar + ";");
		out.write("\n");
		out.write(encodedControlVar + "." + SHOW_H_LINE_PARAM + " = " + showHLine + ";");
		out.write("\n");
		out.write(encodedControlVar + "." + SHOW_V_LINE_PARAM + " = " + showVLine + ";");
		out.write("\n");
		out.write(encodedControlVar + "." + STYLE_CLASS_PARAM + " = \"" + styleClass + "\";");
		out.write("\n");

		//labels and tooltips and messages
		out.write(
			encodedControlVar + "." + PIE_CHART_BUTTON_LABEL_PARAM + " = " + PIE_CHART_BUTTON_LABEL_KEY + ";");
		out.write("\n");
		out.write(
			encodedControlVar + "." + BAR_CHART_BUTTON_LABEL_PARAM + " = " + BAR_CHART_BUTTON_LABEL_KEY + ";");
		out.write("\n");
		out.write(
			encodedControlVar + "." + LINE_CHART_BUTTON_LABEL_PARAM + " = " + LINE_CHART_BUTTON_LABEL_KEY + ";");
		out.write("\n");
		out.write(encodedControlVar + "." + SHARE_TOOLTIP_LABEL_PARAM + " = " + SHARE_TOOLTIP_LABEL_KEY + ";");
		out.write("\n");
		out.write(encodedControlVar + "." + SERIES_TOOLTIP_LABEL_PARAM + " = " + SERIES_TOOLTIP_LABEL_KEY + ";");
		out.write("\n");
		out.write(encodedControlVar + "." + LABEL_TOOLTIP_LABEL_PARAM + " = " + LABEL_TOOLTIP_LABEL_KEY + ";");
		out.write("\n");
		out.write(encodedControlVar + "." + VALUE_TOOLTIP_LABEL_PARAM + " = " + VALUE_TOOLTIP_LABEL_KEY + ";");
		out.write("\n");
		out.write(encodedControlVar + "." + PIE_ERROR_MSG_PARAM + " = " + PIE_ERROR_MSG_KEY + ";");
		out.write("\n");
		out.write(encodedControlVar + "." + SHOW_ALL_COMBO_OPTION_PARAM + " = " + SHOW_ALL_COMBO_OPTION_KEY + ";");
		out.write("\n");
		
		out.write(
				encodedAdapterVar
				+ ".accessibleTextContainer = document.getElementById(\""
				+ encodedAdapterVar
				+ "_AccessibleText\");");
		out.write("\n");

		//bind and show graph		
		out.write(encodedAdapterVar + ".bind();");
		out.write("\n");
		out.write(encodedControlVar + ".ShowGraph();");
		out.write("\n");

	//	out.write("</script>");
		out.write("\n");
	//	out.write("<script type=\"text/javascript\">\n");
		out.write("ODCProgressBar.endRenderControl(graphdraw);\n");
//		out.write("</script>\n");
		Streamer.trace.Header().println("Exiting GraphDrawEmitter...");
	}

	/**
	 * Sets the flag that determines if data values are displayed next to the pie section in a pie chart,
	 * next to the bar in a bar chart and next to the point in a line chart.
	 * <p>
	 * @param showLabel	If true, the data values are displayed.
	 */
	public void setShowLabel(boolean showLabel) {
		this.showLabel = showLabel;
	}

	/**
	 * Sets the label attribute. The label attribute is the attribute on the data object that is
	 * used as the source of values for plotting the x axis.
	 * <p>
	 * @param labelAttributeName	The label attribute name
	 */
	public void setLabelAttributeName(String labelAttributeName) {
		this.labelAttributeName = labelAttributeName;
	}

	/**
	 * Sets the grouped flag. If true, the data values in each series are grouped by the grouping operation
	 * specified for that series. If false, the data values are plotted individually.
	 * <p>
	 * @param grouped	If true, data values are grouped
	 */
	public void setGrouped(boolean grouped) {
		this.grouped = grouped;
	}

	/**
	 * Adds a data series to the list maintained by the emitter. A data series corresponds to a series
	 * plotted in the y axis.
	 * <p>
	 * @param id				The id
	 * @param seriesName		The series name
	 * @param attributeName		The attribute name for the data values
	 * @param groupOperation	The group operation. This is "None" by default.
	 */
	public void addDataSeries(String id, String seriesName, String attributeName, String groupOperation) {
		DataSeries series = new DataSeries(id, seriesName, attributeName, groupOperation);
		seriesList.add(series);
	}

	StringBuffer seriesNames = new StringBuffer();
	StringBuffer attNames = new StringBuffer();
	StringBuffer ops = new StringBuffer();

	private void processDataSeries() {
		for (int i = 0; i < seriesList.size(); ++i) {
			DataSeries cTemp = (DataSeries) seriesList.get(i);
			String op = cTemp.getGroupOperation();
			if (op == null || op.equals(""))
				op = "None";

			if (i == 0) {
				seriesNames = seriesNames.append("\"").append(cTemp.getSeriesName()).append("\"");
				attNames = attNames.append("\"").append(cTemp.getAttributeName()).append("\"");
				ops = ops.append("\"").append(op).append("\"");
			} else {
				seriesNames = seriesNames.append(",\"").append(cTemp.getSeriesName()).append("\"");
				attNames = attNames.append(",\"").append(cTemp.getAttributeName()).append("\"");
				ops = ops.append(",\"").append(op).append("\"");
			}
		}
	}

	/**
	 * Sets the JavaScript fragment that creates the converter to be used for formatting the labels.
	 * <p>
	 * @param labelConverterScript	The label converter script
	 */
	public void setLabelConverterScript(String labelConverterScript) {
		this.labelConverterScript = labelConverterScript;
		if (this.labelConverterScript.equals(""))
			this.labelConverterScript = "\"\"";
	}

	/**
	 * Sets the JavaScript fragment that creates the converter to be used for formatting the data values.
	 * <p>
	 * @param dataConverterScript	The data converter script
	 */
	public void setDataConverterScript(String dataConverterScript) {
		this.dataConverterScript = dataConverterScript;
		if (this.dataConverterScript.equals(""))
			this.dataConverterScript = "\"\"";
	}

	protected class DataSeries {
		public DataSeries(String id, String seriesName, String attributeName, String groupOperation) {
			this.id = id;
			this.seriesName = seriesName;
			this.attributeName = attributeName;
			this.groupOperation = groupOperation;
		}

		public String getId() {
			return id;
		}

		public String getSeriesName() {
			return seriesName;
		}

		public String getAttributeName() {
			return attributeName;
		}

		public String getGroupOperation() {
			if (groupOperation != null)
				groupOperation = groupOperation.toUpperCase();
			return groupOperation;
		}

		private String id;
		private String seriesName;
		private String attributeName;
		private String groupOperation;
	}

	/**
	 * Sets the type of formatting to be done on the labels. If not specified, it defaults to NO_FORMAT, 
	 * where no formatting is done. The values allowed are NO_FORMAT, DATETIME_FORMAT, NUMBER_FORMAT and MASK_FORMAT. 
	 * <p>
	 * @param format	The type of formatting to be done
	 */
	public void setLabelFormat(String format) {
		this.labelFormat = format;
	}

	/**
	 * Sets the type of formatting to be done on the data values. If not specified, it defaults to NO_FORMAT, 
	 * where no formatting is done. The values allowed are NO_FORMAT, DATETIME_FORMAT and NUMBER_FORMAT.
	 * <p>
	 * @param format	The type of formatting to be done
	 */
	public void setDataFormat(String format) {
		this.dataFormat = format;
	}

	private String id;
	private String modelName;
	private String eObjectId;
	private String eReferenceName;
	private String width;
	private String height;
	private String firstChart;
	private boolean showPie;
	private boolean showBar;
	private boolean showLine;
	private String yAxisDivisions;
	private String title;
	private String xAxisTitle;
	private String yAxisTitle;
	private boolean showLegend;
	private boolean showLabel;
	private boolean splitYAxis;
	private boolean grouped;
	private ArrayList seriesList = new ArrayList();
	private String labelAttributeName;
	private boolean showHBar;
	private boolean showHLine;
	private boolean showVLine;
	private String styleClass;	

	private String labelFormat = GraphDrawEmitter.NO_FORMAT;
	private String dataFormat = GraphDrawEmitter.NO_FORMAT;

	private String labelConverterScript = "\"\"";
	private String dataConverterScript = "\"\"";

	private String LABEL_ATTRIBUTE_PARAM = "labelAttributeName";
	private String SERIES_NAMES_PARAM = "dataSeriesNames";
	private String VALUE_ATTRIBUTES_PARAM = "dataAttributeNames";
	private String GROUP_OPS_PARAM = "groupOperations";

	private String LABEL_FORMAT_PARAM = "labelFormat";
	private String DATA_FORMAT_PARAM = "dataFormat";

	private String LABEL_CONVERTER_PARAM = "labelConverter";
	private String DATA_CONVERTER_PARAM = "dataConverter";

	private String CHART_TITLE_PARAM = "chartTitle";
	private String X_AXIS_TITLE_PARAM = "xAxisTitle";
	private String Y_AXIS_TITLE_PARAM = "yAxisTitle";
	private String SHOW_LEGEND_PARAM = "showLegend";
	private String SHOW_LABEL_PARAM = "showLabel";
	private String SHOW_PIE_PARAM = "showPie";
	private String SHOW_BAR_PARAM = "showBar";
	private String SHOW_LINE_PARAM = "showLine";
	private String FIRST_CHART_PARAM = "defaultChartType";
	private String Y_AXIS_DIVISIONS_PARAM = "yAxisDivisions";
	private String SPLIT_Y_AXIS_PARAM = "splitYAxis";
	private String GROUPED_PARAM = "grouped";

	private String PIE_CHART_BUTTON_LABEL_PARAM = "lblPie";
	private String BAR_CHART_BUTTON_LABEL_PARAM = "lblBar";
	private String LINE_CHART_BUTTON_LABEL_PARAM = "lblLine";

	private String SHARE_TOOLTIP_LABEL_PARAM = "tooltipShare";
	private String SERIES_TOOLTIP_LABEL_PARAM = "tooltipSeries";
	private String LABEL_TOOLTIP_LABEL_PARAM = "tooltipLabel";
	private String VALUE_TOOLTIP_LABEL_PARAM = "tooltipValue";

	private String PIE_ERROR_MSG_PARAM = "msgPieChartError";
	private String SHOW_ALL_COMBO_OPTION_PARAM = "lblShowAll";

	private String PIE_CHART_BUTTON_LABEL_KEY = "chartdraw_lblPie";
	private String BAR_CHART_BUTTON_LABEL_KEY = "chartdraw_lblBar";
	private String LINE_CHART_BUTTON_LABEL_KEY = "chartdraw_lblLine";

	private String SHARE_TOOLTIP_LABEL_KEY = "chartdraw_tooltipShare";
	private String SERIES_TOOLTIP_LABEL_KEY = "chartdraw_tooltipSeries";
	private String LABEL_TOOLTIP_LABEL_KEY = "chartdraw_tooltipLabel";
	private String VALUE_TOOLTIP_LABEL_KEY = "chartdraw_tooltipValue";

	private String PIE_ERROR_MSG_KEY = "chartdraw_msgPieChartError";
	
	private String SHOW_ALL_COMBO_OPTION_KEY = "chartdraw_lblShowAll";

	public static final String DATA_PREFIX = "data";
	public static final String LABEL_PREFIX = "label";

	public static final String NO_FORMAT = "STRING";
	public static final String DATETIME_FORMAT = "DATETIME";
	public static final String NUMBER_FORMAT = "NUMBER";
	public static final String MASK_FORMAT = "MASK";

	public static final String SHOW_H_BAR_PARAM = "showHorizontalLinesBar";
	public static final String SHOW_H_LINE_PARAM = "showHorizontalLinesLine";
	public static final String SHOW_V_LINE_PARAM = "showVerticalLinesLine";
	public static final String STYLE_CLASS_PARAM = "styleClass";
}