/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//
//OCO Source Materials
//
//PID xxxx-xxx
//
// Copyright IBM Corp.  2003
//
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////

package com.ibm.odcb.jrender.misc;


import java.lang.reflect.Constructor;
import java.lang.reflect.Modifier;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.impl.EObjectImpl;

/**
 * @author ldh
 *
 * Various helper methods around Java types
 * The OBF deals with many type systems and needs strong mapping functions. This class provides simple static methods to do all sorts of validation
 * and mappings. For instance, if you are introspecting a class, or dealing with a Class object of some kind, you can do a getName() on it and ask a bunch of questions such
 * as:
 *      Is that type a basic Java type (String, Integer, Boolean...)?
 *      Is it an Array of primitive types ([B, [C ...) ?
 *      Is it an Array of basic types ([Ljava.lang.String;, [Ljava.lang.Boolean; ...) ?
 *      What is the simple JavaScript type mapping for that Java type (java has many "integer" types, but the simplified type system only has one: int)
 *      What is the EDataType for a JavaType and vice-versa
 */
public class TypesUtil
 {
   //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
   //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
   // List of all basic types, and a simple mapping to a reduces type system of string, char, boolean. float, int and date
   //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

   public static class TypeInfo
    {
      public TypeInfo(Class JavaClass, String JavaArrayType, String SimpleType, String EDataType, String EDataGet, String EDataSet, char OutputType, boolean Primitive)
       {
         _JavaClass     = JavaClass;
         _JavaType      = JavaClass.getName();
         _JavaArrayType = JavaArrayType;
         _SimpleType    = SimpleType   ;
         _EDataType     = EDataType    ;
         _EDataGet      = EDataGet     ;
         _EDataSet      = EDataSet     ;
         _OutputType    = OutputType   ;   
         _Primitive     = Primitive    ;
       }
      public Class   _JavaClass    ;
      public String  _JavaType     ;
      public String  _JavaArrayType;
      public String  _SimpleType   ;
      public String  _EDataType    ;
      public String  _EDataGet     ;
      public String  _EDataSet     ;
      public char    _OutputType   ;
      public boolean _Primitive    ;

      public static final char _CHAR_OUT   = 'C';  // data type should be output as a char
      public static final char _STRING_OUT = 'S';  // data type should be output as a string
      public static final char _NUMBER_OUT = 'N';  // data type should be output as a number
      public static final char _DATE_OUT   = 'D';  // data type should be output as a date
      public static final char _BOOL_OUT   = 'B';  // data type should be output as a boolean
      
      // LDH-NOTE: Is the default like a String, or none of the above (i.e. ' ')?
      public static final char _DEFAULT_OUT= ' ';  // data type should be output as default
    }
    
////#IF# SDO    
//	  public static final String DATE_ETYPE = "EDate";
////#END#

////#IF# WDO    
    public static final String DATE_ETYPE = "WDODate";
////#END#

////#IF# SDO    
//	  public static final String BIG_DECIMAL_ETYPE = "EBigDecimal";
////#END#

////#IF# WDO    
    public static final String BIG_DECIMAL_ETYPE = "WDOBigDecimal";
////#END#
    
    private static final String[] DECIMAL_TYPES = {"java.lang.Double","java.lang.Float","java.math.BigDecimal","double","float"};
    
    public static boolean isDecimalType(String type)
    {
    	if (type == null) return false;
    	for (int i=0; i<DECIMAL_TYPES.length; i++)
    	{
    		if (type.equals(DECIMAL_TYPES[i]))
    			return true;
    	}
    	return false;
    }
       
   protected static final TypeInfo BASIC_TYPES[] =
      { new TypeInfo(java.lang.String.class    , "[Ljava.lang.String;"    , "string" , "EString"       , "getString"    , "setString"    , TypeInfo._STRING_OUT, false)
       ,new TypeInfo(java.lang.Character.class , "[Ljava.lang.Character;" , "char"   , "ECharObject"   , "getChar"      , "setChar"      , TypeInfo._CHAR_OUT  , false)
       ,new TypeInfo(java.lang.Boolean.class   , "[Ljava.lang.Boolean;"   , "boolean", "EBooleanObject", "getBoolean"   , "setBoolean"   , TypeInfo._BOOL_OUT  , false)
       ,new TypeInfo(java.lang.Double.class    , "[Ljava.lang.Double;"    , "float"  , "EDoubleObject" , "getDouble"    , "setDouble"    , TypeInfo._NUMBER_OUT, false)
       ,new TypeInfo(java.lang.Float.class     , "[Ljava.lang.Float;"     , "float"  , "EFloatObject"  , "getFloat"     , "setFloat"     , TypeInfo._NUMBER_OUT, false)
       ,new TypeInfo(java.lang.Integer.class   , "[Ljava.lang.Integer;"   , "int"    , "EIntegerObject", "getInt"       , "setInt"       , TypeInfo._NUMBER_OUT, false)
       ,new TypeInfo(java.lang.Long.class      , "[Ljava.lang.Long;"      , "int"    , "ELongObject"   , "getLong"      , "setLong"      , TypeInfo._NUMBER_OUT, false)
       ,new TypeInfo(java.lang.Short.class     , "[Ljava.lang.Short;"     , "int"    , "EShortObject"  , "getShort"     , "setShort"     , TypeInfo._NUMBER_OUT, false)
       ,new TypeInfo(java.lang.Byte.class      , "[Ljava.lang.Byte;"      , "int"    , "EByteObject"   , "getByte"      , "setByte"      , TypeInfo._NUMBER_OUT, false)
       ,new TypeInfo(java.util.Date.class      , "[Ljava.util.Date;"      , "date"   , DATE_ETYPE       , "getDate"      , "setDate"      , TypeInfo._DATE_OUT  , false)
       ,new TypeInfo(java.math.BigDecimal.class, "[Ljava.math.BigDecimal;", "float"  , BIG_DECIMAL_ETYPE , "getBigDecimal", "setBigDecimal", TypeInfo._NUMBER_OUT, false)
       ,new TypeInfo(char.class                , "[C"                     , "char"   , "EChar"         , "getChar"      , "setChar"      , TypeInfo._CHAR_OUT  , true )
       ,new TypeInfo(boolean.class             , "[Z"                     , "boolean", "EBoolean"      , "getBoolean"   , "setBoolean"   , TypeInfo._BOOL_OUT  , true )
       ,new TypeInfo(double.class              , "[D"                     , "float"  , "EDouble"       , "getDouble"    , "setDouble"    , TypeInfo._NUMBER_OUT, true )
       ,new TypeInfo(float.class               , "[F"                     , "float"  , "EFloat"        , "getFloat"     , "setFloat"     , TypeInfo._NUMBER_OUT, true )
       ,new TypeInfo(int.class                 , "[I"                     , "int"    , "EInt"          , "getInt"       , "setInt"       , TypeInfo._NUMBER_OUT, true )
       ,new TypeInfo(long.class                , "[J"                     , "int"    , "ELong"         , "getLong"      , "setLong"      , TypeInfo._NUMBER_OUT, true )
       ,new TypeInfo(short.class               , "[S"                     , "int"    , "EShort"        , "getShort"     , "setShort"     , TypeInfo._NUMBER_OUT, true )
       ,new TypeInfo(byte.class                , "[B"                     , "int"    , "EByte"         , "getByte"      , "setByte"      , TypeInfo._NUMBER_OUT, true )
//LDH-NOTE: Beware anyone who wishes to uncomment this line. Technically speaking, we do not support Object properties,
//        especially not as basic properties (attributes, not references). If you are uncommenting this, it's a hack 
//        to make something else work, but it is very likely it may have strong unintended consequences. 
//	   ,new TypeInfo("java.lang.Object", "[Ljava.lang.Object;", "string"   , "EJavaObject"   , "getString"   , TypeInfo._STRING_OUT  , false)
//       ,new TypeInfo(java.util.Calendar.class  , "[Ljava.util.Calendar;"  , "date"   , DATE_ETYPE       , "getDate"      , "setDate"      , TypeInfo._DATE_OUT  , false)
     };

   // returns true if the Java type passed in is a basic type (i.e. String, Integer, Double, Boolean, ...., int, double, boolean ...)
   public static boolean isBasicJavaType(String Type)
    {
      for (int i = 0; i < BASIC_TYPES.length; ++i)
       {
         if (BASIC_TYPES[i]._JavaType.equals(Type) == true)
          return true;
       }
      return false;
    }

   // returns true if the Java type passed is a primitive type (int, boolean...) or not (Integer, Boolean ...) based on the boolean flag
   public static boolean isPrimitiveJavaType(String Type)
    {
      for (int i = 0; i < BASIC_TYPES.length; ++i)
       {
         if (BASIC_TYPES[i]._Primitive == true && BASIC_TYPES[i]._JavaType.equals(Type) == true)
          return true;
       }
      return false;
    }

   // returns true if the Java type passed in is an array of basic types
   public static boolean isArrayOfBasicJavaType(String Type)
    {
      for (int i = 0; i < BASIC_TYPES.length; ++i)
       {
         if (BASIC_TYPES[i]._JavaArrayType.equals(Type) == true)
          return true;
       }
      return false;
    }

   // returns true if the Java type passed in is an array of complex (non basic) types
   public static boolean isArrayOfComplexJavaType(String Type)
    {
      return isArray(Type) == true && !isArrayOfBasicJavaType(Type);
    }

   // returns true if the Java type passed in is an array of basic primitive types or not based on the boolean flag
   public static boolean isArrayOfBasicJavaType(String Type, boolean Primitive)
    {
      for (int i = 0; i < BASIC_TYPES.length; ++i)
       {
         if (BASIC_TYPES[i]._Primitive == Primitive && BASIC_TYPES[i]._JavaArrayType.equals(Type) == true)
          return true;
       }
      return false;
    }

   // returns true if the Java type passed in is an array of non primitive types (i.e., any class)
   public static boolean isArrayOfNonPrimitiveJavaType(String Type)
    {
      return isArray(Type) == true && Type.equals("[Ljava.lang.Object;") == false;
    }

   public static boolean isArray(String Type)
    {
      return Type.startsWith("[L") == true && Type.endsWith(";") == true;
    }

   public static String getComplexJavaTypeFromArrayType(String Type)
    {
      return Type.substring(2, Type.length()-1).replace('$','.')+"[]";
    }
   public static String getBaseJavaTypeFromArrayType(String Type)
    {
      return Type.substring(2, Type.length()-1).replace('$','.');
    }

   // returns true if the string passed in is a valid EData type (EString, EBoolean, EBoolenaObject...)
   public static boolean isBasicEDataType(String Type)
    {
      for (int i = 0; i < BASIC_TYPES.length; ++i)
       {
         if (BASIC_TYPES[i]._EDataType.equals(Type) == true)
          return true;
       }
      return false;
    }
    
   public static boolean isJavaObjectType(String Type, boolean ArrayToo)
    {
      return    Type.equals(Object.class.getName()) == true
             || ArrayToo == true && Type.equals(Object[].class.getName()) == true
             ;
    } 

   // returns a simple type (string, char, int, float, boolean, date) based on the "primitive" Java type passed in 
   // (Integer, Byte, Short, Double, Float, int, byte, boolean ...). This method also returns simplidifed types if 
   // arrays of Java primitive types arre passed in. This simplified type system is targeted for JavaScript.
   public static String getSimplifiedTypeFromJavaType(String Type)
    {
      for (int i = 0; i < BASIC_TYPES.length; ++i)
       {
         if (BASIC_TYPES[i]._JavaType.equals(Type) == true)
          return BASIC_TYPES[i]._SimpleType;
         if (BASIC_TYPES[i]._JavaArrayType.equals(Type) == true)
          return BASIC_TYPES[i]._SimpleType;
       }
      return null;
    }

   // returns the EData type based on the passed in Java type
   public static String getEDataTypeFromJavaType(String Type, boolean Many)
    {
      for (int i = 0; i < BASIC_TYPES.length; ++i)
       {
         if (BASIC_TYPES[i]._JavaType.equals(Type) == true || (Many == true && BASIC_TYPES[i]._JavaArrayType.equals(Type) == true) )
          return BASIC_TYPES[i]._EDataType;
       }
      return null;
    }
   public static String getEDataTypeFromJavaType(String Type)
    {
      return getEDataTypeFromJavaType(Type, false);
    }
  
   // returns the Java type based on the passed in EData type
   public static String getJavaTypeFromEDataType(String Type, boolean Many)
    {
      for (int i = 0; i < BASIC_TYPES.length; ++i)
       {
         if (BASIC_TYPES[i]._EDataType.equals(Type) == true)
          return Many == true ? BASIC_TYPES[i]._JavaArrayType : BASIC_TYPES[i]._JavaType;
       }
      return null;
    }

   // returns "char" or "java.lang.String" is string passed in is "[C" or "[Ljava.lang.String;" respectively for all Java basic types    
   public static String getJavaTypeFromArrayType(String Type)
    {
      for (int i = 0; i < BASIC_TYPES.length; ++i)
       {
         if (BASIC_TYPES[i]._JavaArrayType.equals(Type) == true)
          return BASIC_TYPES[i]._JavaType;
       }
      return null;
    }

   // returns "char" or "String" is string passed in is "[C" or "[Ljava.lang.String;" respectively for all Java basic types    
   public static String getShortJavaTypeFromArrayType(String Type)
    {
      String Str = getJavaTypeFromArrayType(Type);
      if (Str == null)
       return null;
      int i = Str.lastIndexOf('.');
      if (i != -1)
       {
         Str = Str.substring(i+1);
       }
      return Str;
    }
    
   // Returns the output type for a given Java type
   public static char getOutputTypeFromJavaType(String Type)
    {
      for (int i = 0; i < BASIC_TYPES.length; ++i)
       {
         if (BASIC_TYPES[i]._JavaType.equals(Type) == true || BASIC_TYPES[i]._JavaArrayType.equals(Type) == true)
          return BASIC_TYPES[i]._OutputType;
       }
      return TypeInfo._DEFAULT_OUT;
    }
   
   // Returns the output type for a given EData type
   public static char getOutputTypeFromEDataType(String Type)
    {
      for (int i = 0; i < BASIC_TYPES.length; ++i)
       {
         if (BASIC_TYPES[i]._EDataType.equals(Type) == true)
          return BASIC_TYPES[i]._OutputType;
       }
      return TypeInfo._DEFAULT_OUT;
    }

   public static final String BASIC_ETYPE_PREFIX = "ecore:EDataType http://www.eclipse.org/emf/2002/Ecore#//";
   public static final String CUSTOM_ETYPE_PREFIX = "#//";
   protected static final String UNSUPPORTED_TYPE_HEADER = "__UNSUPPORTED_TYPE__";
   protected static final String UNKNOWN_TYPE_HEADER = "__UNKNOWN_TYPE__";
   protected static final String[][] BAD_TYPES_MAP  = { { "WDOByteArray"  , UNSUPPORTED_TYPE_HEADER+"WDOByteArray" },
   	                                                    { "EByteArray"    , UNSUPPORTED_TYPE_HEADER+"EByteArray"   }
                                                      };
   public static boolean isUnsupportedSDOType(String eType)
   {
   	 boolean bad = false;
   	 for (int i=0; i<BAD_TYPES_MAP.length; i++)
   	 {
   	 	if (BAD_TYPES_MAP[i][0].equals(eType))
   	 	  return true;
   	 }
   	 return bad;   	
   }
   public static final String WDO_ETYPE_PREFIX = "ecore:EDataType http://ibm.com/wdo#//";
   protected static final String[][] SDO_TO_WDO_ETYPE_MAP = {{"EDate", "WDODate"},{"EBigDecimal", "WDOBigDecimal"},{"EByteArray","WDOByteArray"}};
   
   public static boolean hasWDOEquivalent(String etype)
   {
   	boolean yes = false;
   	for (int i=0; i<SDO_TO_WDO_ETYPE_MAP.length; i++)
   	{
   		if (SDO_TO_WDO_ETYPE_MAP[i][0].equals(etype))
   		{
   		  yes = true;
   		  break;
		}
   	}
   	return yes;
   }
   
   public static String getWDOEquivalent(String etype)
	{
	 String type = null;
	 for (int i=0; i<SDO_TO_WDO_ETYPE_MAP.length; i++)
	 {
		 if (SDO_TO_WDO_ETYPE_MAP[i][0].equals(etype))
		 {
		   type = SDO_TO_WDO_ETYPE_MAP[i][1];
		   break;
		 }
	 }
	 return type;
	}
	
	public static String getFullEType(String eType, boolean useWDO)
	{	
		if (TypesUtil.isBasicEDataType(eType) || TypesUtil.isUnsupportedSDOType(eType))
		{
			if (useWDO && TypesUtil.hasWDOEquivalent(eType))
			{
				eType = TypesUtil.getWDOEquivalent(eType);
				eType = TypesUtil.WDO_ETYPE_PREFIX+eType;
			}
			else
			  eType = TypesUtil.BASIC_ETYPE_PREFIX+eType;
		}
		else
		{
			eType = TypesUtil.CUSTOM_ETYPE_PREFIX+eType;
		}	
		return eType;
	}
   
   // for a given EData type, returns the getter method name for a WDO object
   public static String getEDataGet(String ETypeName)
    {
      for (int i = 0; i < BASIC_TYPES.length; ++i)
       {
         if (BASIC_TYPES[i]._EDataType.equals(ETypeName) == true)
          return BASIC_TYPES[i]._EDataGet;
       }
      for (int i = 0; i < BAD_TYPES_MAP.length; ++i)
       {
         if (BAD_TYPES_MAP[i][0].equals(ETypeName) == true)
          return BAD_TYPES_MAP[i][1];
       }
      return UNKNOWN_TYPE_HEADER +ETypeName;
    }

   // for a given EData type, returns the getter method name for a WDO object
   public static String getEDataSet(String ETypeName)
    {
      for (int i = 0; i < BASIC_TYPES.length; ++i)
       {
         if (BASIC_TYPES[i]._EDataType.equals(ETypeName) == true)
          return BASIC_TYPES[i]._EDataSet;
       }
      for (int i = 0; i < BAD_TYPES_MAP.length; ++i)
       {
         if (BAD_TYPES_MAP[i][0].equals(ETypeName) == true)
          return BAD_TYPES_MAP[i][1];
       }
      return UNKNOWN_TYPE_HEADER+ETypeName;
    }

   public static boolean isUnsupportedType(String Type)
    {
		return Type.startsWith(UNSUPPORTED_TYPE_HEADER) || Type.startsWith(UNKNOWN_TYPE_HEADER);
    }


   //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
   //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
   // List of types considered lists
   //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
   public static final Class COLLECTION_TYPES[] = { Collection.class, List.class, Iterator.class };    

   private static final boolean isGeneralListInternal(String Type, boolean ArraysToo)
   throws ClassNotFoundException
    {
      if (isBasicJavaType(Type) == true)
       return false;
      // LDH-NOTE: Test for Array. Previously, i got the class and called isArray() on it, but in debug mode under WSAD,
      //          the JVM crashes there for some reason. So, this new test will short-circuit this behavior. 
      if (Type.charAt(0) == '[')
       return ArraysToo;
      Class C = ODCClassLoader.getClass(Type);
      for (int i = 0; i < COLLECTION_TYPES.length; ++i)
       {
         if (COLLECTION_TYPES[i].isAssignableFrom(C) == true)
          return true;
       }
      return false;
    }
   public static boolean isGeneralList(String Type)
   throws ClassNotFoundException
    {
      return isGeneralListInternal(Type, true);
    }
   public static boolean isGeneralListNoArrays(String Type)
   throws ClassNotFoundException
    {
      return isGeneralListInternal(Type, false);
    }

   public static boolean isCollection(String Type)
   throws ClassNotFoundException
    {
      return Collection.class.isAssignableFrom(ODCClassLoader.getClass(Type));
    }


   //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
   //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
   // These are property names that we automatically filter out. The getClass() in every Java class, and 
   // the typical getBoolean(), getInt() etc.... which are not bean properties 
   //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
   public static final String[] INVALID_PROPERTY_NAMES = { "char" 
                                                         , "boolean" 
                                                         , "double" 
                                                         , "float" 
                                                         , "int" 
                                                         , "long" 
                                                         , "short" 
                                                         , "byte"
                                                         , "class" 
                                                         , "character" 
                                                         , "integer"
                                                         , "object"
                                                         };
                                                     
   public static boolean isValidProperty(String Name)
    {
      for (int i = 0; i < INVALID_PROPERTY_NAMES.length; ++i)
       {
         if (Name.equals(INVALID_PROPERTY_NAMES[i]) == true)
          return false;
       }
      return true;
    }
    
   
   //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
   //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
   // Obtaining a Type from an EClass is sometimes harder than it should. getEType().getName() often returns null 
   //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
   public static String getRealETypeName(EClassifier E)
    {      
      // simple Test first. This should work all the time, but for some unclear reasons as of yet, sometimes, it returns null
      String Type = E.getName();

       if (E.getInstanceClassName() != null) {
	        	// map instance class name to an EdataType
	        	String instanceClassName = E.getInstanceClassName();
	          for (int i = 0; i < BASIC_TYPES.length; ++i)
	          {
	            if (instanceClassName.equals(BASIC_TYPES[i]._JavaType))
	             return BASIC_TYPES[i]._EDataType;
	          }
	        }

      if (Type != null)
       return Type;
       
      // If there is an eProxy URL, use that then
      if (E.eIsProxy() == true && E instanceof EObjectImpl)
       {
         Type = ((EObjectImpl) E).eProxyURI().toString();
       }
       
      // Last resort, try the toString method. This returns something of the form:
      //      org.eclipse.emf.ecore.impl.EDataTypeImpl@7ea53ceb (eProxyURI: http://www.eclipse.org/emf/2002/Ecore#//ECharObject)
      // So we need to parse some things out
      if (Type == null)
       {
         Type = E.toString();
         int i = Type.indexOf("(eProxyURI: ");
         if (i != -1)
          {
            Type = Type.substring(i+"(eProxyURI: ".length(), Type.length()-1);
          }
       }

      // If we have some string at this time, we can check 
      if (Type != null)
       {
         for (int i = 0; i < BASIC_TYPES.length; ++i)
          {
            if (Type.endsWith("//"+BASIC_TYPES[i]._EDataType) == true)
             return BASIC_TYPES[i]._EDataType;
          }
       }
      return null;
    }
    
   public static Class getPrimitiveTypeClass(String PrimitiveJavaType)
    {
      for (int i = 0; i < BASIC_TYPES.length; ++i)
       {
         if (BASIC_TYPES[i]._Primitive == true && BASIC_TYPES[i]._JavaType.equals(PrimitiveJavaType) == true)
          return BASIC_TYPES[i]._JavaClass;
       }
      return null;
    }
   
   public static boolean isInstantiable(Class testClass){
       if(testClass.isInterface() || Modifier.isAbstract(testClass.getModifiers())){
           return false;
       }
     Constructor[] constructors = testClass.getConstructors();
     if(constructors.length == 0){
         return false;
     }
     boolean hasDefaultConstructor = false;
     for (int i = 0; i < constructors.length; i++) {
        Constructor tempConstructor = constructors[i];
        if(tempConstructor.getParameterTypes().length == 0){
            hasDefaultConstructor = true;
            break;
        }
    }
       return hasDefaultConstructor;
   }
 }
