/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2004 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.odcb.utilities;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.util.ArrayList;

import com.ibm.odcb.jrender.misc.FileUtil;
import com.ibm.odcb.jrender.misc.Streamer;

/**
 * @author ldh
 */
public class PreProcessor
 {
   public static class Params
    {
      public File       _RootDir  ;
      public boolean    _Recurse  = false;
      public ArrayList  _FileMasks= new ArrayList();
      public ArrayList  _Defines  = new ArrayList();
      public boolean    _Backup   = true;
      
      public boolean Parse(String[] args)
       {
         int i = 0;
         if (i >= args.length)
          {
            System.err.println("The utility was called without any parameters.");
            return false;
          }
         _RootDir = new File(args[i]);
         
         if (++i >= args.length)  // No more parameters, it's OK. Defaults have already been initialized.
          {
            return true;
          }
         if (args[i].equalsIgnoreCase("-nobackup") == true)
          {
            _Backup = false;
            if (++i >= args.length) //  no more parameters, fine. Defaults have already been initialized
             {
               return true;
             }
          }
         if (args[i].equalsIgnoreCase("-r") == true)
          {
            _Recurse = true;
            if (++i >= args.length) //  no more parameters, fine. Defaults have already been initialized
             {
               return true;
             }
          }
         if (args[i].equalsIgnoreCase("-e") == true)
          {
            while (++i < args.length) // parse the tokens for file extensions
             {
               if (args[i].equalsIgnoreCase("-d") == true) // starting the next round of parameters?
                break;
               _FileMasks.add(args[i]);
             }
            if (i >= args.length) // Here, either we ran out of parameters, or we hit a -d. If we ran out, we are fine.
             {
               return true;
             }
          }

         if (args[i].equalsIgnoreCase("-d") == true)
          {
            while (++i < args.length)
             {
               _Defines.add(args[i]);
             }
          }

         return true;
       }
    }
    
   public static final String TOKEN_IF   = "//#IF#"  ;
   public static final String TOKEN_ELSE = "//#ELSE#";
   public static final String TOKEN_END  = "//#END#" ;
   
   public static void main(String[] args)
    {
      Streamer.setLogLevel(3);
      Streamer.status.Header().println("PreProcessor <root_dir> {-nobackup} {-r} { -e <file_extension>+ ) {-d <token>+ }\n"
                        +"  Call this utility to process all files with the given extensions (-e) from a root directory,\n"
                        +" and on down recursively if -r is specified. The utility will rewrite files based on the defined\n"
                        +" tokens. if -nobackup is specified, then modified files will not be backed up.\n"
                        +"  In order to make the code compilable in Java (or any language where '//' is a line comment) and\n"
                        +" not bother developers using this facility, you must comment one conditional block, and the\n"
                        +" utility will handle the code accordingly. Make sure to use the '//' comments only at the begining\n"
                        +" of the line for each line in the commented block. For example, let's consider the code sample\n"
                        +" below:\n"
                        +"     "+TOKEN_IF+" TOTO\n"
                        +"         Foo();\n"
                        +"         Boo();\n"
                        +"         Goo();\n"
                        +"     "+TOKEN_ELSE+"\n"
                        +"     //    Far();\n"
                        +"     //    Bar();\n"
                        +"     //    Gar();\n"
                        +"     "+TOKEN_END+"\n"
                        +"  If TOTO is defined, the code won't change. If TOTO is not defined though, the code will be\n"
                        +" rewritten as:\n"
                        +"     "+TOKEN_IF+" TOTO\n"
                        +"//         Foo();\n"
                        +"//         Boo();\n"
                        +"//         Goo();\n"
                        +"     "+TOKEN_ELSE+"\n"
                        +"         Far();\n"
                        +"         Bar();\n"
                        +"         Gar();\n"
                        +"     "+TOKEN_END+"\n"
                        +"  The markers such as '"+TOKEN_IF+"' must be on a line with nothing else on them except the\n"
                        +" token, and starting with white spaces. '"+TOKEN_ELSE+"' blocks are optional. Additionally,\n"
                        +" make sure to also not have normal comments at all in any blocks, or the commenting and\n"
                        +" uncommenting algorithm will mess up your code: ALL commented lines in an included block\n"
                        +" will ne uncommented, and ALL uncommented lines in a non-included block will be commented.\n\n");

      Params P = new Params();
      if (P.Parse(args) == true)
       {
         Process(P);
       }
      Streamer.status.Header().println("Utility has finished processing");
      Streamer.FlushThread();
    }
    
   public static boolean Process(Params P)
    {
      return FileUtil.FileSystemTraverser(P._RootDir, new FileUtil.ExtensionFilter(P._FileMasks, true)
                                                    , new IfElseEnd_Processor(P._Defines, P._Backup));
    }
    
   static class IfElseEnd_Processor implements FileUtil.FileProcessor
    {
      public IfElseEnd_Processor(ArrayList Defines, boolean Backup)
       {
         _Defines = Defines;
         _Backup  = Backup;
       }
      protected ArrayList _Defines;
      protected boolean   _Backup ;
      
      public boolean Process(File F)
      throws Exception
       {
         if (F.isDirectory() == true)
          return true;
         Streamer.debug.Header().println("Checking the file "+F.getPath());
         BufferedReader In = new BufferedReader(new FileReader(F));
         ArrayList FileLines = new ArrayList(57);
         String Line = In.readLine();
         int LineNumber = 1;
         int IfElseEndBlock = 0;
         boolean IncludeBlock=true;
         boolean ChangesMade = false;
         while (Line != null)
          {
            String TrimmedLine = Line.trim();
            if (isLineTokenned(TrimmedLine, TOKEN_IF) == true) // Check our IF marker
             {
               if (IfElseEndBlock != 0) // If we are already in a block, we have a syntax error
                {
                  Streamer.error.Header().println("File '"+F.getPath()+"', line '"+LineNumber
                                                 +"': found '"+TOKEN_IF+"' nested within another '"+TOKEN_IF+"/"+TOKEN_END+"' block.");
                  return false;  
                }
               IfElseEndBlock = 1; // Mark that we have started an IF block
               IncludeBlock = isTokenDefined(Line); // Check the token specified if it is in the list and set the Include flag
             }
            else if (isLineTokenned(TrimmedLine, TOKEN_ELSE) == true) // Check our ELSE marker
             {
               if (IfElseEndBlock != 1) // If we are not right after an IF, then we have a syntax error
                {
                  Streamer.error.Header().println("File '"+F.getPath()+"', line '"+LineNumber
                                                 +"': found '"+TOKEN_ELSE+"' not after '"+TOKEN_IF+"'.");
                  return false;  
                }
               IfElseEndBlock = 2;
               IncludeBlock = !IncludeBlock; // We are in an ELSE block now, so we reverse the Include flag
             }
            else if (isLineTokenned(TrimmedLine, TOKEN_END) == true) // Check our END marker
             {
               if (IfElseEndBlock == 0) // If we are not after an IF or an ELSE, then we have a syntax error
                {
                  Streamer.error.Header().println("File '"+F.getPath()+"', line '"+LineNumber
                                                 +"': found '"+TOKEN_END+"' not after '"+TOKEN_IF+"' or '"+TOKEN_END+"'.");
                  return false;  
                }
               IfElseEndBlock = 0; // We end our block, so we mark that
               IncludeBlock = true; // Out of a block, so everything is included
             }
            else if (IfElseEndBlock != 0) // We are in a block, so we may have to comment/uncomment lines in here.
             {
               String UpdatedLine = IncludeBlock==true ? UncommentLine(Line, TrimmedLine) : CommentLine(Line, TrimmedLine);
               if (UpdatedLine != null)
                {
                  ChangesMade = true;
                  Line = UpdatedLine;
                }
             }
            FileLines.add(Line);
            Line = In.readLine();
            ++LineNumber;
          }
         In.close(); // close the file.
         
         if (ChangesMade == true)
          {
            boolean ReadOnly = !F.canWrite();
            if (_Backup == true)
             {
               Streamer.debug.Header().println("  --> Making a backup...");
               String FileName = F.getAbsolutePath();            
               File BackupF = new File(FileName+".bak");
               if (BackupF.exists() == true)
                {
                  BackupF.delete();
                }
               F.renameTo(BackupF);
               F = new File(FileName);
             }
            else
             {
               F.delete();
             }            
            Streamer.debug.Header().println("  --> Rewriting...");
            FileOutputStream Out = new FileOutputStream(F); // Reopen the file but for output
            for (int i = 0; i < FileLines.size(); ++i)
             {
               Line = (String) FileLines.get(i); // Write the modified content back.
               Out.write(Line.getBytes());
               Out.write(FileUtil.NEWLINE.getBytes());
             } 
            Out.close();
            if (ReadOnly == true)
             F.setReadOnly();
            Streamer.debug.Header().println("  --> Finished processing the file");
          }
         
         return true;
       }
       
      public boolean isTokenDefined(String Line)
       {
         Line = Line.substring(Line.indexOf(TOKEN_IF)+TOKEN_IF.length()).trim();
         for (int i = 0; i < _Defines.size(); ++i)
          {
            if (Line.equals((String) _Defines.get(i)) == true)
             return true;
          }
         return false;
       } 

      public String UncommentLine(String Line, String TrimmedLine)
       {
         if (TrimmedLine.startsWith("//") == true)
          {
            int i = Line.indexOf("//");
            return Line.substring(0, i)+Line.substring(i+2);
          }
         return null; // Do nothing, Line is not commented
       }

      public String CommentLine(String Line, String TrimmedLine)
       {
         if (TrimmedLine.startsWith("//") == false)
          {
            return "//"+Line;
          }
         return null;
       }

     // A line is Tokenned if it contains the Token and nothing before it
     public boolean isLineTokenned(String Line, String Token)
      {
        int i = Line.indexOf(Token);
        if (i == -1)
         return false;
        while (i >= 0)
         {
           char C = Line.charAt(i--);
           if (C != '/' && Character.isWhitespace(C) == false)
            return false;
         }
        return true;
      }
    }
 }
